@@ -1,10 +1,7 @@
 use binread::BinRead;
 use std::io::{Cursor, Seek, SeekFrom, Write};
 
-use crate::{
-    anim::*, formats::mesh::*, matl::*, shdr::*, skel::*, Offset, Ptr, RelPtr64, SsbhArray,
-    SsbhByteBuffer, SsbhFile, SsbhWrite,
-};
+use crate::{Offset, Ptr, RelPtr64, SsbhArray, SsbhByteBuffer, SsbhFile, SsbhWrite};
 
 fn round_up(value: u64, n: u64) -> u64 {
     // Find the next largest multiple of n.
@@ -25,57 +22,6 @@ fn write_relative_offset<W: Write + Seek>(writer: &mut W, data_ptr: &u64) -> std
     Ok(())
 }
 
-macro_rules! ssbh_write_c_enum_impl {
-    ($enum_type:ident,$underlying_type:ident) => {
-        impl SsbhWrite for $enum_type {
-            fn ssbh_write<W: std::io::Write + std::io::Seek>(
-                &self,
-                writer: &mut W,
-                _data_ptr: &mut u64,
-            ) -> std::io::Result<()> {
-                let value = *self as $underlying_type;
-                let bytes = value.to_le_bytes();
-                writer.write_all(&bytes)?;
-                Ok(())
-            }
-
-            fn size_in_bytes(&self) -> u64 {
-                std::mem::size_of::<$underlying_type>() as u64
-            }
-
-            fn alignment_in_bytes() -> u64 {
-                std::mem::align_of::<$underlying_type>() as u64
-            }
-        }
-    };
-}
-
-// TODO: These can be derived at some point.
-// TODO: It may be better to move these next to their definition.
-ssbh_write_c_enum_impl!(ShaderType, u32);
-
-ssbh_write_c_enum_impl!(CompressionType, u8);
-ssbh_write_c_enum_impl!(TrackType, u8);
-ssbh_write_c_enum_impl!(AnimType, u64);
-
-ssbh_write_c_enum_impl!(AttributeDataTypeV8, u32);
-ssbh_write_c_enum_impl!(AttributeDataTypeV10, u32);
-ssbh_write_c_enum_impl!(AttributeUsageV8, u32);
-ssbh_write_c_enum_impl!(AttributeUsageV9, u32);
-ssbh_write_c_enum_impl!(RiggingType, u32);
-ssbh_write_c_enum_impl!(DrawElementType, u32);
-
-ssbh_write_c_enum_impl!(BlendFactor, u32);
-ssbh_write_c_enum_impl!(WrapMode, u32);
-ssbh_write_c_enum_impl!(CullMode, u32);
-ssbh_write_c_enum_impl!(FillMode, u32);
-ssbh_write_c_enum_impl!(MinFilter, u32);
-ssbh_write_c_enum_impl!(MagFilter, u32);
-ssbh_write_c_enum_impl!(FilteringType, u32);
-ssbh_write_c_enum_impl!(ParamId, u64);
-
-ssbh_write_c_enum_impl!(BillboardType, u8);
-
 fn write_array_header<W: Write + Seek>(
     writer: &mut W,
     data_ptr: &mut u64,
@@ -88,7 +34,7 @@ fn write_array_header<W: Write + Seek>(
     if count == 0 {
         write_u64(writer, 0u64)?;
     } else {
-        write_relative_offset(writer, &data_ptr)?;
+        write_relative_offset(writer, data_ptr)?;
     }
 
     write_u64(writer, count as u64)?;
@@ -181,7 +127,7 @@ fn write_rel_ptr_aligned_specialized<
             writer.seek(SeekFrom::Start(*data_ptr))?;
 
             // Allow custom write functions for performance reasons.
-            write_t(&value, writer, data_ptr)?;
+            write_t(value, writer, data_ptr)?;
 
             // Point the data pointer past the current write.
             // Types with relative offsets will already increment the data pointer.
@@ -246,8 +192,16 @@ impl<P: Offset, T: SsbhWrite + BinRead<Args = ()>> SsbhWrite for Ptr<P, T> {
                 // Calculate the absolute offset.
                 *data_ptr = round_up(*data_ptr, alignment);
 
-                let offset = P::try_from(*data_ptr)
-                    .map_err(|_| std::io::Error::new(std::io::ErrorKind::Other, "oh no!"))?;
+                let offset = P::try_from(*data_ptr).map_err(|_| {
+                    std::io::Error::new(
+                        std::io::ErrorKind::Other,
+                        format!(
+                            "Failed to convert offset {} to a pointer with {} bytes.",
+                            data_ptr,
+                            std::mem::size_of::<P>()
+                        ),
+                    )
+                })?;
                 P::ssbh_write(&offset, writer, data_ptr)?;
 
                 // Write the data at the specified offset.
@@ -357,8 +311,9 @@ mod tests {
     // but following these rules creates 1:1 export for all formats except NRPD.
 
     use super::*;
-    use crate::{Ptr16, Ptr32, Ptr64, SsbhEnum64, SsbhString, SsbhString8, hex_bytes};
+    use crate::{Ptr16, Ptr32, Ptr64, SsbhEnum64, SsbhString, SsbhString8};
     use binread::BinRead;
+    use hexlit::hex;
 
     #[test]
     fn write_ptr16() {
@@ -368,7 +323,7 @@ mod tests {
         let mut data_ptr = 0;
         value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
 
-        assert_eq!(*writer.get_ref(), hex_bytes("0200 05"));
+        assert_eq!(writer.into_inner(), hex!("0200 05"));
         assert_eq!(3, data_ptr);
     }
 
@@ -380,7 +335,7 @@ mod tests {
         let mut data_ptr = 0;
         value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
 
-        assert_eq!(*writer.get_ref(), hex_bytes("04000000 05"));
+        assert_eq!(writer.into_inner(), hex!("04000000 05"));
         assert_eq!(5, data_ptr);
     }
 
@@ -392,7 +347,7 @@ mod tests {
         let mut data_ptr = 0;
         value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
 
-        assert_eq!(*writer.get_ref(), hex_bytes("00000000"));
+        assert_eq!(writer.into_inner(), hex!("00000000"));
         assert_eq!(4, data_ptr);
     }
 
@@ -404,7 +359,7 @@ mod tests {
         let mut data_ptr = 0;
         value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
 
-        assert_eq!(*writer.get_ref(), hex_bytes("08000000 00000000 05"));
+        assert_eq!(writer.into_inner(), hex!("08000000 00000000 05"));
         assert_eq!(9, data_ptr);
     }
 
@@ -417,7 +372,7 @@ mod tests {
         let mut data_ptr = 0;
         value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
 
-        assert_eq!(*writer.get_ref(), hex_bytes("08000000 00000000 05"));
+        assert_eq!(writer.into_inner(), hex!("08000000 00000000 05"));
         assert_eq!(9, data_ptr);
     }
 
@@ -430,7 +385,7 @@ mod tests {
         let mut data_ptr = 0;
         value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
 
-        assert_eq!(*writer.get_ref(), hex_bytes("08000000 00000000 05000000"));
+        assert_eq!(writer.into_inner(), hex!("08000000 00000000 05000000"));
         assert_eq!(12, data_ptr);
     }
 
@@ -442,7 +397,7 @@ mod tests {
         let mut data_ptr = 0;
         value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
 
-        assert_eq!(*writer.get_ref(), hex_bytes("00000000 00000000"));
+        assert_eq!(writer.into_inner(), hex!("00000000 00000000"));
         assert_eq!(8, data_ptr);
     }
 
@@ -455,8 +410,8 @@ mod tests {
         value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
 
         assert_eq!(
-            *writer.get_ref(),
-            hex_bytes(
+            writer.into_inner(),
+            hex!(
                 "08000000 00000000 
                  08000000 00000000 
                  07000000"
@@ -474,8 +429,8 @@ mod tests {
         value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
 
         assert_eq!(
-            *writer.get_ref(),
-            hex_bytes("08000000 00000000 73636F75 74657231 53686170 6500")
+            writer.into_inner(),
+            hex!("08000000 00000000 73636F75 74657231 53686170 6500")
         );
         // The data pointer should be aligned to 4.
         assert_eq!(24, data_ptr);
@@ -489,7 +444,7 @@ mod tests {
         let mut data_ptr = 0;
         value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
 
-        assert_eq!(*writer.get_ref(), hex_bytes("08000000 00000000 00000000"));
+        assert_eq!(writer.into_inner(), hex!("08000000 00000000 00000000"));
         // The data pointer should be aligned to 4.
         assert_eq!(12, data_ptr);
     }
@@ -503,8 +458,8 @@ mod tests {
         value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
 
         assert_eq!(
-            *writer.get_ref(),
-            hex_bytes("08000000 00000000 73636F75 74657231 53686170 6500")
+            writer.into_inner(),
+            hex!("08000000 00000000 73636F75 74657231 53686170 6500")
         );
         // The data pointer should be aligned to 4.
         assert_eq!(24, data_ptr);
@@ -524,8 +479,8 @@ mod tests {
         // Check that the relative offsets point past the array.
         // Check that string data is aligned to 4.
         assert_eq!(
-            *writer.get_ref(),
-            hex_bytes(
+            writer.into_inner(),
+            hex!(
                 "10000000 00000000 02000000 00000000
                  10000000 00000000 20000000 00000000
                  6C657965 735F6579 655F6D61 72696F5F 
@@ -545,8 +500,8 @@ mod tests {
 
         // Null and empty arrays seem to use 0 offset and 0 length.
         assert_eq!(
-            *writer.get_ref(),
-            hex_bytes("00000000 00000000 00000000 00000000")
+            writer.into_inner(),
+            hex!("00000000 00000000 00000000 00000000")
         );
         assert_eq!(16, data_ptr);
     }
@@ -560,8 +515,8 @@ mod tests {
         value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
 
         assert_eq!(
-            *writer.get_ref(),
-            hex_bytes("10000000 00000000 05000000 00000000 01020304 05")
+            writer.into_inner(),
+            hex!("10000000 00000000 05000000 00000000 01020304 05")
         );
         assert_eq!(21, data_ptr);
     }
@@ -574,7 +529,7 @@ mod tests {
         let mut data_ptr = 0;
         value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
 
-        assert_eq!(*writer.get_ref(), hex_bytes("01020304 05"));
+        assert_eq!(writer.into_inner(), hex!("01020304 05"));
         assert_eq!(5, data_ptr);
     }
 
@@ -588,8 +543,8 @@ mod tests {
 
         // Null and empty arrays seem to use 0 offset and 0 length.
         assert_eq!(
-            *writer.get_ref(),
-            hex_bytes("00000000 00000000 00000000 00000000")
+            writer.into_inner(),
+            hex!("00000000 00000000 00000000 00000000")
         );
         assert_eq!(16, data_ptr);
     }
@@ -614,8 +569,8 @@ mod tests {
 
         // Check that the pointers don't overlap.
         assert_eq!(
-            *writer.get_ref(),
-            hex_bytes(
+            writer.into_inner(),
+            hex!(
                 "10000000 00000000 20000000 00000000 
                  5254565F 4652414D 455F4255 46464552 
                  5F434F50 59000000 46425F46 52414D45 
@@ -633,8 +588,8 @@ mod tests {
         value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
 
         assert_eq!(
-            *writer.get_ref(),
-            hex_bytes("08000000 00000000 426C656E 64537461 74653000")
+            writer.into_inner(),
+            hex!("08000000 00000000 426C656E 64537461 74653000")
         );
         // The data pointer should be aligned to 8.
         assert_eq!(24, data_ptr);
@@ -649,8 +604,8 @@ mod tests {
         value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
 
         assert_eq!(
-            *writer.get_ref(),
-            hex_bytes("08000000 00000000 00000000 00000000")
+            writer.into_inner(),
+            hex!("08000000 00000000 00000000 00000000")
         );
         // The data pointer should be aligned to 8.
         assert_eq!(16, data_ptr);
@@ -665,8 +620,8 @@ mod tests {
         value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
 
         assert_eq!(
-            *writer.get_ref(),
-            hex_bytes("08000000 00000000 426C656E 64537461 74653000")
+            writer.into_inner(),
+            hex!("08000000 00000000 426C656E 64537461 74653000")
         );
         // The data pointer should be aligned to 8.
         assert_eq!(24, data_ptr);
@@ -693,8 +648,8 @@ mod tests {
         value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
 
         assert_eq!(
-            *writer.get_ref(),
-            hex_bytes("10000000 00000000 01000000 00000000 0000803F")
+            writer.into_inner(),
+            hex!("10000000 00000000 01000000 00000000 0000803F")
         );
     }
 
@@ -710,8 +665,8 @@ mod tests {
         value.ssbh_write(&mut writer, &mut data_ptr).unwrap();
 
         assert_eq!(
-            *writer.get_ref(),
-            hex_bytes("10000000 00000000 02000000 00000000 05000000")
+            writer.into_inner(),
+            hex!("10000000 00000000 02000000 00000000 05000000")
         );
     }
 }
