@@ -1,11 +1,12 @@
 use crate::SsbhArray;
 use crate::SsbhByteBuffer;
 use crate::SsbhString;
-#[cfg(feature = "serde")]
-use serde::{Deserialize, Serialize};
+use binread::BinRead;
+use modular_bitfield::prelude::*;
 use ssbh_write::SsbhWrite;
 
-use binread::BinRead;
+#[cfg(feature = "serde")]
+use serde::{Deserialize, Serialize};
 
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[derive(BinRead, Debug, SsbhWrite)]
@@ -13,7 +14,7 @@ pub struct AnimTrackV2 {
     pub name: SsbhString,
     pub flags: TrackFlags,
     pub frame_count: u32,
-    pub unk3: u32,
+    pub unk_flags: UnkTrackFlags, // flags?
     pub data_offset: u32,
     pub data_size: u64,
 }
@@ -28,7 +29,7 @@ pub struct AnimNode {
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[derive(BinRead, Debug, SsbhWrite)]
 pub struct AnimGroup {
-    pub anim_type: AnimType,
+    pub group_type: GroupType,
     pub nodes: SsbhArray<AnimNode>,
 }
 
@@ -96,7 +97,7 @@ pub struct AnimHeaderV20 {
     pub unk1: u16, // always 1?
     pub unk2: u16, // always 3?
     pub name: SsbhString,
-    pub animations: SsbhArray<AnimGroup>,
+    pub groups: SsbhArray<AnimGroup>,
     pub buffer: SsbhByteBuffer,
 }
 
@@ -111,11 +112,12 @@ pub struct AnimHeaderV21 {
     pub unk1: u16, // always 1?
     pub unk2: u16, // always 3?
     pub name: SsbhString,
-    pub animations: SsbhArray<AnimGroup>,
+    pub groups: SsbhArray<AnimGroup>,
     pub buffer: SsbhByteBuffer,
     pub unk_data: UnkData,
 }
 
+// TODO: Is this interpolation data?
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[derive(BinRead, Debug, SsbhWrite)]
 pub struct UnkData {
@@ -126,17 +128,18 @@ pub struct UnkData {
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[derive(BinRead, Debug, SsbhWrite)]
 pub struct UnkItem1 {
-    pub unk1: u64,
-    pub unk2: SsbhArray<UnkSubItem>,
+    pub unk1: u64,                   // TODO: Always 2?
+    pub unk2: SsbhArray<UnkSubItem>, // TODO: Always (0, final_frame_index)?
 }
 
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[derive(BinRead, Debug, SsbhWrite)]
 pub struct UnkItem2 {
-    pub unk1: SsbhString,
-    pub unk2: SsbhArray<UnkSubItem>,
+    pub unk1: SsbhString,            // TODO: node name?
+    pub unk2: SsbhArray<UnkSubItem>, // TODO: (frame start, frame end)?
 }
 
+// TODO: These appear to be start and end frame indices.
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[derive(BinRead, Debug, SsbhWrite)]
 pub struct UnkSubItem {
@@ -153,9 +156,26 @@ pub struct TrackFlags {
     pub compression_type: CompressionType,
 }
 
+#[bitfield(bits = 32)]
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
-#[derive(BinRead, Debug, Clone, Copy, PartialEq, Eq)]
+#[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
+#[derive(Debug, BinRead, Clone, Copy)]
+#[br(map = Self::from_bytes)]
+pub struct UnkTrackFlags {
+    unk1: bool, // TODO: unk1?
+    disable_rotation: bool,
+    disable_scale: bool,
+    disable_compensate_scale: bool,
+    #[skip]
+    __: B28,
+}
+
+ssbh_write::ssbh_write_modular_bitfield_impl!(UnkTrackFlags, 4);
+
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[derive(BinRead, Debug, SsbhWrite, Clone, Copy, PartialEq, Eq)]
 #[br(repr(u8))]
+#[ssbhwrite(repr(u8))]
 pub enum TrackType {
     Transform = 1,
     UvTransform = 2,
@@ -166,20 +186,25 @@ pub enum TrackType {
 }
 
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
-#[derive(BinRead, Debug, Clone, Copy, PartialEq, Eq)]
+#[derive(BinRead, Debug, SsbhWrite, Clone, Copy, PartialEq, Eq)]
 #[br(repr(u8))]
+#[ssbhwrite(repr(u8))]
 pub enum CompressionType {
     Direct = 1,
+
+    // TODO: This can be used with non transform tracks for version 2.0 and 2.1.
+    // ex: assist/metroid/model/body/c00/model.nuanmb
     ConstTransform = 2,
+
     Compressed = 4,
     Constant = 5,
 }
 
-// TODO: Rename to group type.
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
-#[derive(BinRead, Debug, Clone, Copy)]
+#[derive(BinRead, Debug, SsbhWrite, Clone, Copy, PartialEq, Eq)]
 #[br(repr(u64))]
-pub enum AnimType {
+#[ssbhwrite(repr(u64))]
+pub enum GroupType {
     Transform = 1,
     Visibility = 2,
     Material = 4,
