@@ -96,7 +96,8 @@ pub enum ParamV15 {
     #[br(pre_assert(data_type == 7u64))]
     Unk7(Color4f),
 
-    /// A string value used to store texture names for texture parameters.
+    /// A string value used to store texture file names.
+    /// Examples: `"../../textures/cos_149000_02"`, `"/common/shader/sfxPBS/default_Params"`, `"#replace_cubemap"`, `"asf_ashley_col"`.
     #[br(pre_assert(data_type == 11u64))]
     MatlString(SsbhString),
 
@@ -133,7 +134,8 @@ pub enum ParamV16 {
     #[br(pre_assert(data_type == 7u64))]
     Unk7(Color4f),
 
-    /// A string value used to store texture names for texture parameters.
+    /// A string value used to store texture file names.
+    /// Examples: `"../../textures/cos_149000_02"`, `"/common/shader/sfxPBS/default_Params"`, `"#replace_cubemap"`, `"asf_ashley_col"`.    
     #[br(pre_assert(data_type == 11u64))]
     MatlString(SsbhString),
 
@@ -153,12 +155,13 @@ pub enum ParamV16 {
 /// The possible values for [param_id](struct.MatlAttribute.html.#structfield.param_id).
 /// Not all values are used by Smash Ultimate's shaders.
 /// For up to date documentation, see the [Material Parameters](https://github.com/ScanMountGoat/Smush-Material-Research/blob/master/Material%20Parameters.md) page on Github.
-// Sorted by occurrence count in descending order to improve matching performance.
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, Clone, Copy, PartialEq)]
+#[derive(BinRead, Debug, SsbhWrite, Clone, Copy, PartialEq, Eq)]
 #[br(repr(u64))]
+#[ssbhwrite(repr(u64))]
 pub enum ParamId {
+    // Sorted by Smash Ultimate's occurrence count in descending order to improve matching performance.
     BlendState0 = 280,
     RasterizerState0 = 291,
     CustomVector8 = 160,
@@ -410,7 +413,7 @@ pub enum ParamId {
     CustomFloat14 = 206,
     CustomFloat15 = 207,
     CustomFloat16 = 208,
-    // The following values are unused.
+    // The following values are unused for Smash Ultimate.
     CustomInteger0 = 212,
     CustomInteger1 = 213,
     CustomInteger2 = 214,
@@ -531,8 +534,9 @@ pub enum ParamId {
 /// The possible values for [fill_mode](struct.MatlRasterizerStateV16.html.#structfield.fill_mode).
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, Clone, Copy, PartialEq)]
+#[derive(BinRead, Debug, SsbhWrite, Clone, Copy, PartialEq, Eq)]
 #[br(repr(u32))]
+#[ssbhwrite(repr(u32))]
 pub enum FillMode {
     Line = 0,
     Solid = 1,
@@ -541,8 +545,9 @@ pub enum FillMode {
 /// The possible values for [cull_mode](struct.MatlRasterizerStateV16.html.#structfield.cull_mode).
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, Clone, Copy, PartialEq)]
+#[derive(BinRead, Debug, SsbhWrite, Clone, Copy, PartialEq)]
 #[br(repr(u32))]
+#[ssbhwrite(repr(u32))]
 pub enum CullMode {
     Back = 0,
     Front = 1,
@@ -574,8 +579,9 @@ pub struct MatlRasterizerStateV16 {
 
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, Clone, Copy, PartialEq)]
+#[derive(BinRead, Debug, SsbhWrite, Clone, Copy, PartialEq, Eq)]
 #[br(repr(u32))]
+#[ssbhwrite(repr(u32))]
 pub enum WrapMode {
     Repeat = 0,
     ClampToEdge = 1,
@@ -585,8 +591,9 @@ pub enum WrapMode {
 
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, Clone, Copy, PartialEq)]
+#[derive(BinRead, Debug, SsbhWrite, Clone, Copy, PartialEq, Eq)]
 #[br(repr(u32))]
+#[ssbhwrite(repr(u32))]
 pub enum MinFilter {
     Nearest = 0,
     LinearMipmapLinear = 1,
@@ -595,8 +602,9 @@ pub enum MinFilter {
 
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, Clone, Copy, PartialEq)]
+#[derive(BinRead, Debug, SsbhWrite, Clone, Copy, PartialEq, Eq)]
 #[br(repr(u32))]
+#[ssbhwrite(repr(u32))]
 pub enum MagFilter {
     Nearest = 0,
     Linear = 1,
@@ -605,8 +613,9 @@ pub enum MagFilter {
 
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, Clone, Copy, PartialEq)]
+#[derive(BinRead, Debug, SsbhWrite, Clone, Copy, PartialEq, Eq)]
 #[br(repr(u32))]
+#[ssbhwrite(repr(u32))]
 pub enum FilteringType {
     Default = 0,
     Default2 = 1,
@@ -643,8 +652,9 @@ pub struct MatlUvTransform {
 
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, Clone, Copy, PartialEq)]
+#[derive(BinRead, Debug, SsbhWrite, Clone, Copy, PartialEq, Eq)]
 #[br(repr(u32))]
+#[ssbhwrite(repr(u32))]
 pub enum BlendFactor {
     Zero = 0,
     One = 1,
@@ -665,11 +675,14 @@ pub enum BlendFactor {
 #[derive(BinRead, Debug, Clone, Copy, PartialEq, SsbhWrite)]
 pub struct MatlBlendStateV15 {
     pub unk1: u64,
-    pub unk2: u64,
-    pub unk3: u64,
-    pub unk4: u64,
-    pub unk5: u64,
+    pub unk2: u32,
+    pub unk3: u32,
+    pub unk4: u32,
+    pub unk5: u32,
     pub unk6: u64,
+    pub unk7: u32,
+    pub unk8: u32,
+    pub unk9: u32,
 }
 
 /// Determines the alpha blending settings to use when rendering.
