@@ -7,8 +7,9 @@ use ssbh_write::SsbhWrite;
 #[repr(u32)]
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
-#[derive(BinRead, Debug, Clone, Copy)]
+#[derive(BinRead, Debug, SsbhWrite, Clone, Copy)]
 #[br(repr(u32))]
+#[ssbhwrite(repr(u32))]
 pub enum ShaderType {
     Vertex = 0,
     Geometry = 3,
@@ -16,6 +17,7 @@ pub enum ShaderType {
     Compute = 5,
 }
 
+// TODO: The binary seems to contain names for uniforms.
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[cfg_attr(feature = "arbitrary", derive(arbitrary::Arbitrary))]
 #[derive(BinRead, Debug, SsbhWrite)]
@@ -23,10 +25,10 @@ pub struct Shader {
     pub name: SsbhString,
     pub shader_type: ShaderType,
     pub unk3: u32,
-    pub shader_binary: SsbhByteBuffer,
+    pub shader_binary: SsbhByteBuffer, // TODO: Additional parsing for this?
+    pub binary_size: u64,
     pub unk4: u64,
     pub unk5: u64,
-    pub binary_size: u64,
 }
 
 /// A compiled shader container.
